# -*- encoding: utf-8 -*-
##############################################################################
#
#    Avanzosc - Avanced Open Source Consulting
#    Copyright (C) 2011 - 2012 Avanzosc <http://www.avanzosc.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import wizard
import pooler
import netsvc
from osv import fields, osv
from tools.translate import _

class wiz_stock_merge_picking(wizard.interface):
    
    def get_price(self, cr, uid, id, stock, context):
        pool = pooler.get_pool(cr.dbname)
        delivery_objs = pool.get('delivery.carrier')
        total = 0
        weight = 0
        volume = 0

        for line in stock.move_lines:
            carrier = False
            if not line.product_id:
                continue
            for delivery in delivery_objs.browse(cr, uid, delivery_objs.search(cr, uid, [])):
                if line.product_id.id == delivery.product_id.id:
                    carrier = True
            if not carrier:
                qty = line.product_qty
                weight += (line.product_id.weight or 0.0) * qty
                volume += (line.product_id.volume or 0.0) * qty
                
        return self.get_price_from_picking(cr, uid, id, weight, volume, context)
    
    def get_price_from_picking(self, cr, uid, id, weight, volume, context):
        pool = pooler.get_pool(cr.dbname)
        grid = pool.get('delivery.grid').browse(cr, uid, id)
        
        price = 0.0
        ok = False

        for line in grid.line_ids:
            price_dict = {'price': price, 'volume':volume, 'weight': weight, 'wv':volume*weight}
            test = eval(line.type+line.operator+str(line.max_value), price_dict)
            if test:
                price = line.list_price
                if line.price_type=='variable':
                    price *= price_dict[line.variable_factor]
                ok = True
                break
        if not ok:
            raise osv.except_osv(_('No price avaible !'), _('No line matched this order in the choosed delivery grids !'))
        return price
    
    def get_sale_disc(self, cr, uid, stock_pick):
        pool = pooler.get_pool(cr.dbname)
        discount = 0
#        if stock_pick.origin == False:
#            return 0
#        origin = stock_pick.origin[stock_pick.origin.find(":")+1:stock_pick.origin.find(" ")+1]
#        if origin == '':
#            origin = stock_pick.origin
        
#        sale_order_obj = pool.get('sale.order')
#        ids = sale_order_obj.search(cr, uid, [('name','like', origin)])
        for sale in stock_pick.sale_ids:
            discount = sale.global_disc
#        if ids == []:
#            discount = 0
#        else:
#            sale_order = sale_order_obj.browse(cr, uid, ids)[0]
#            discount = sale_order.global_disc

        return discount
    
    def find_delivery(self, cr, uid, data, stock_pick, carriers):
        for stock_move in stock_pick.move_lines:
            for carrier in carriers:
                if stock_move.product_id.id == carrier.product_id.id:
                    return carrier
        return False
    
    def test_picking(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        stock_pick_obj = pool.get('stock.picking').browse(cr, uid, data['ids'])
        delivery_objs = pool.get('delivery.carrier')
        carriers = delivery_objs.browse(cr, uid, delivery_objs.search(cr, uid, []))
        
        if (len(stock_pick_obj) == 1) or (len(stock_pick_obj) == None):
            raise osv.except_osv(_('Selection error'), _('Please select more than one package !'))
        else:
            partner = stock_pick_obj[0].address_id.id
            global_disc = self.get_sale_disc(cr, uid, stock_pick_obj[0])
            carrier = self.find_delivery(cr, uid, data, stock_pick_obj[0], carriers)
        if stock_pick_obj[0].type != 'out':
            raise osv.except_osv(_('Selection error'), _('The packages have to be outgoing packages !'))   
        if ((stock_pick_obj[0].state != 'draft') and (stock_pick_obj[0].state != 'confirmed') and (stock_pick_obj[0].state != 'assigned')):
            raise osv.except_osv(_('Selection error'), _('The package state have to be "Draft", "Confirmed" or "Available" !'))
        else:
            state = stock_pick_obj[0].state
        if stock_pick_obj[0].invoice_state != '2binvoiced':
            raise osv.except_osv(_('Selection error'), _('The package invoice state have to be "To Be invoiced" !'))
        else:
            invoice_state = stock_pick_obj[0].invoice_state
            
        for stock_pick in stock_pick_obj:
            if stock_pick.address_id.id != partner:
                raise osv.except_osv(_('Selection error'), _('Please select picking with the same partner !'))
            if stock_pick.state != state:
                raise osv.except_osv(_('Selection error'), _('Please select picking with the same state !'))
            if stock_pick.invoice_state != invoice_state:
                raise osv.except_osv(_('Selection error'), _('Please select the same invoice state packages !'))
            if global_disc != self.get_sale_disc(cr, uid, stock_pick):
                raise osv.except_osv(_('Selection error'), _('Please select picking with the same global discount !'))
            carrier2 = self.find_delivery(cr, uid, data, stock_pick, carriers)
            if carrier and carrier2:
                if carrier != carrier2:
                    raise osv.except_osv(_('Selection error'), _('Please select picking with the same delivery carrier !'))
#            elif carrier:
#                if not carrier2:
#                    raise osv.except_osv('Selection error', 'Some products has not delivery carrier !')
#            else:
#                if carrier2:
#                    raise osv.except_osv('Selection error', 'Some products has not delivery carrier !')           
        self.merge_picking(cr, uid, data, stock_pick_obj, context)   
                
        return {}
    
    def merge_picking(self, cr, uid, data, stock_pick_obj, context):
        wf_service = netsvc.LocalService("workflow")  
        pool = pooler.get_pool(cr.dbname)
        stock_pick_objs= pool.get('stock.picking')
        carrier_obj = pool.get('delivery.carrier')
        grid_objs = pool.get('delivery.grid')
        sale_objs = pool.get('sale.order')
        sale_line_objs = pool.get('sale.order.line')
        proc_objs = pool.get('mrp.procurement')
        carriers = carrier_obj.browse(cr, uid, carrier_obj.search(cr, uid, []))
        sales = []
        origins = []
        notes = ''
        moveLines = []
        moveCarrier = []
        pick_ids = []
        
        for stock_pick in stock_pick_obj:
            if stock_pick.id == stock_pick_obj[0].id:
                origin = stock_pick.origin
                date = stock_pick.date
            else:
                if stock_pick.origin != False:
                    origin += ' {name}'.format(name=stock_pick.origin)
                if stock_pick.date < date:
                    date = stock_pick.date
            if stock_pick.note != False:
                notes += '{note}\n'.format(note=stock_pick.note)
            carrier = self.find_delivery(cr, uid, data, stock_pick, carriers)
            if carrier:
                for move_line in stock_pick.move_lines:
                    if move_line.product_id.id != carrier.product_id.id:     
                        moveLines.append(move_line.id)
                    elif move_line.picking_id.id == stock_pick_obj[0].id:
                        moveCarrier = move_line
                        stock_picking_carrier = carrier
                    else:
                        proc_objs.write(cr, uid, [move_line.sale_line_id.procurement_id.id], {'state': 'draft'})
                        proc_objs.unlink(cr, uid, [move_line.sale_line_id.procurement_id.id])
                        sale_line_objs.write(cr, uid, move_line.sale_line_id.id, {'procurement_id': moveCarrier.sale_line_id.procurement_id.id})
            else:
                for move_line in stock_pick.move_lines:
                    moveLines.append(move_line.id)
            pick_ids.append(stock_pick.id)
            for sale in stock_pick.sale_ids:
                sales.append(sale.id)
        
        value = {
            'origin': origin,
            'address_id': stock_pick_obj[0].address_id.id,
            'type': stock_pick_obj[0].type,
            'invoice_state': stock_pick_obj[0].invoice_state,
            'move_type': 'one',
            'date': date,
            'note': notes,
        }
        
        id = stock_pick_objs.create(cr, uid, value)
        pool.get('stock.move').write(cr, uid, moveLines, {'picking_id': id})
        picking = pool.get('stock.picking').browse(cr, uid, id)
        if stock_pick_obj[0].state in ('confirmed', 'assigned'):
            wf_service.trg_validate(uid, 'stock.picking', picking.id, 'button_confirm', cr)
        if moveCarrier:
            grid_ids = grid_objs.browse(cr, uid, grid_objs.search(cr, uid, [('carrier_id', '=', stock_picking_carrier.id)]))
            grid_id = carrier_obj.grid_get(cr, uid, [stock_picking_carrier.id], picking.address_id.id, context)
            if not grid_id:
                raise osv.except_osv(_('Error'), _('No delivery grid found !'))
            price = self.get_price(cr, uid, grid_id, picking, context)
            pool.get('stock.move').write(cr, uid, moveCarrier.id, {'price_unit': price, 'picking_id': id})
        
        stock_pick_objs.unlink(cr, uid, pick_ids)
        pool.get('stock.picking').write(cr, uid, [id], {})
        
        if sales:
            sale_objs.write(cr, uid, sales, {'picking_ids': [(6, 0, [id])]})

        return True
    
    states = {
        'init': {                 
            'actions': [test_picking],
            'result': {
                'type': 'form', 
                'arch' : """<?xml version="1.0"?>
                            <form string="Merge Picking">
                            <separator string="Merged All Customers Picking Goods!!!" />
                            </form>""", 
                'fields' : {}, 
                'state' : [('end', 'Ok')]
            }
        },
    }
wiz_stock_merge_picking('stock.move.picking.merge')

